/*
 *  scst_pcimem.h
 *
 *  Copyright (C) 2004 - 2011 Vladislav Bolkhovitin <vst@vlnb.net>
 *  Copyright (C) 2004 - 2005 Leonid Stoljar
 *  Copyright (C) 2007 Ming Zhang <blackmagic02881 at gmail dot com>
 *  Copyright (C) 2007 Ross Walker <rswwalker at hotmail dot com>
 *  Copyright (C) 2007 - 2010 ID7 Ltd.
 *  Copyright (C) 2010 - 2011 SCST Ltd.
 *
 *  SCSI disk (type 0) dev handler using PCI-memory space (PCIMEM).
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, version 2
 *  of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 */

#ifndef SCST_PCIMEM_H_
#define SCST_PCIMEM_H_

#define PCIMEM_DUMMY

#ifdef PCIMEM_DUMMY

#define USE_PCIMEM_DUMMY

#define TRACE_ORDER	0x80000000

#define MAX_USN_LEN (20 + 1)
#define	DEF_DISK_BLOCKSIZE_SHIFT	9
#define	DEF_DISK_BLOCKSIZE		(1 << DEF_DISK_BLOCKSIZE_SHIFT)

#define DEF_RD_ONLY			0
#define DEF_WRITE_THROUGH		0
#define DEF_NV_CACHE			0
#define DEF_O_DIRECT			0
#define DEF_REMOVABLE			0
#define DEF_THIN_PROVISIONED		0
#define DEF_MEDIA_EXISTS		1

#define DEF_TST				SCST_CONTR_MODE_SEP_TASK_SETS

/*
 * Since we can't control backstorage device's reordering, we have to always
 * report unrestricted reordering.
 */
#define DEF_QUEUE_ALG_WT	SCST_CONTR_MODE_QUEUE_ALG_UNRESTRICTED_REORDER
#define DEF_QUEUE_ALG		SCST_CONTR_MODE_QUEUE_ALG_UNRESTRICTED_REORDER
#define DEF_SWP			0
#define DEF_TAS			0

#define DEF_DSENSE		SCST_CONTR_MODE_FIXED_SENSE

struct scst_cache_data {
	uint8_t opcode;
	uint8_t evpd;
	uint8_t page;
	uint8_t resp_code;
	uint8_t sense_key;
	uint8_t asc;
	uint8_t ascq;
	uint32_t data_len;
	uint8_t *data;
	struct list_head list_entry;
};

struct scst_pcimem_dev {
	uint32_t block_size;
	uint64_t nblocks;
	int block_shift;
	loff_t file_size;

	spinlock_t flags_lock;

	unsigned int rd_only:1;
	unsigned int wt_flag:1;
	unsigned int nv_cache:1;
	unsigned int o_direct_flag:1;
	unsigned int media_changed:1;
	unsigned int prevent_allow_medium_removal:1;
	unsigned int removable:1;
	unsigned int thin_provisioned:1;

	unsigned int media_exists:1;
	unsigned int no_reserve:1;
	unsigned int exmount:1;

	int virt_id;
	char name[16+1];
	uint16_t command_set_version;
	char t10_dev_id[16+8+2];
	char usn[MAX_USN_LEN];

	struct scst_device *dev;
	struct list_head dev_list_entry;

	int devtype;
	int obmbxid;
	int ibmbxid;
	struct semaphore mbx_sem; // for scst_pcimem_dummy
	wait_queue_head_t mbx_waitQ;
	unsigned short curr_sn;
	struct scst_cmd *ipc_active_cmd;
	unsigned long last_updated;
	spinlock_t active_cmdinfo_lock;

	struct list_head cache_list;
	struct semaphore cache_sem;

	struct scst_tgt_dev *reserve_owner;
};

extern unsigned long *get_trace_flag_ptr(void);
extern int pcimem_get_block_shift(struct scst_cmd *cmd);
extern unsigned long pcimem_bus_to_phys(unsigned long address);
extern int pcimem_add_sg_entries(struct scatterlist *sg, int num_sg, unsigned long offset, unsigned long size);

extern int pcimem_parse(struct scst_cmd *cmd);
extern int pcimem_do_job(struct scst_cmd *cmd);
extern int scst_pcimem_allocator_init(struct scst_pcimem_dev *virt_dev);
extern int pcimem_alloc_data_buf(struct scst_cmd *cmd);
extern void pcimem_on_free_cmd(struct scst_cmd *cmd);
extern int pcimem_task_mgmt_fn(struct scst_mgmt_cmd *mcmd, struct scst_tgt_dev *tgt_dev);

extern int pcimem_add_virtual_device(struct scst_pcimem_dev *virt_dev);
extern void pcimem_del_virtual_device(struct scst_pcimem_dev *virt_dev);
extern int pcimem_init_virtual_dev(void);
extern void pcimem_exit_virtual_dev(void);

#endif /* PCIMEM_DUMMY */

#endif /* SCST_PCIMEM_H_ */

/* end of scst_pcimem.h */
